@@ -1,5 +1,5 @@
 //
-// Copyright 2010-2012 Ettus Research LLC
+// Copyright 2010-2012,2014-2015 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,10 +22,15 @@
 #define UHD_USRP_MULTI_USRP_REF_SOURCES_API
 #define UHD_USRP_MULTI_USRP_GET_RATES_API
 #define UHD_USRP_MULTI_USRP_FRONTEND_CAL_API
+#define UHD_USRP_MULTI_USRP_FRONTEND_IQ_AUTO_API
 #define UHD_USRP_MULTI_USRP_COMMAND_TIME_API
 #define UHD_USRP_MULTI_USRP_BW_RANGE_API
 #define UHD_USRP_MULTI_USRP_USER_REGS_API
 #define UHD_USRP_MULTI_USRP_GET_USRP_INFO_API
+#define UHD_USRP_MULTI_USRP_NORMALIZED_GAIN
+#define UHD_USRP_MULTI_USRP_GPIO_API
+#define UHD_USRP_MULTI_USRP_REGISTER_API
+#define UHD_USRP_MULTI_USRP_FILTER_API
 
 #include <uhd/config.hpp>
 #include <uhd/device.hpp>
@@ -35,6 +40,7 @@
 #include <uhd/types/tune_request.hpp>
 #include <uhd/types/tune_result.hpp>
 #include <uhd/types/sensors.hpp>
+#include <uhd/types/filters.hpp>
 #include <uhd/usrp/subdev_spec.hpp>
 #include <uhd/usrp/dboard_iface.hpp>
 #include <boost/shared_ptr.hpp>
@@ -95,6 +101,8 @@ class UHD_API multi_usrp : boost::noncopyable{
 public:
     typedef boost::shared_ptr<multi_usrp> sptr;
 
+    virtual ~multi_usrp(void) = 0;
+
     //! A wildcard motherboard index
     static const size_t ALL_MBOARDS = size_t(~0);
 
@@ -153,6 +161,11 @@ public:
      * If the specified rate is not available, this method will throw.
      * On other devices, this method notifies the software of the rate,
      * but requires the the user has made the necessary hardware change.
+     *
+     * If the device has an 'auto clock rate' setting (e.g. B200, see also
+     * \ref b200_auto_mcr), this will get disabled and the clock rate will be
+     * fixed to \p rate.
+     *
      * \param rate the new master clock rate in Hz
      * \param mboard the motherboard index 0 to M-1
      */
@@ -280,7 +293,7 @@ public:
     /*!
      * Set the clock configuration for the usrp device.
      * DEPRECATED in favor of set time and clock source calls.
-     * This tells the usrp how to get a 10Mhz reference and PPS clock.
+     * This tells the usrp how to get a 10MHz reference and PPS clock.
      * See the documentation for clock_config_t for more info.
      * \param clock_config the clock configuration to set
      * \param mboard which motherboard to set the config
@@ -313,7 +326,7 @@ public:
 
     /*!
      * Set the clock source for the usrp device.
-     * This sets the source for a 10 Mhz reference clock.
+     * This sets the source for a 10 MHz reference clock.
      * Typical options for source: internal, external, MIMO.
      * \param source a string representing the clock source
      * \param mboard which motherboard to set the config
@@ -487,6 +500,34 @@ public:
         return this->set_rx_gain(gain, ALL_GAINS, chan);
     }
 
+    /*!
+     * Set the normalized RX gain value.
+     *
+     * The normalized gain is a value in [0, 1], where 0 is the
+     * smallest gain value available, and 1 is the largest, independent
+     * of the device. In between, gains are linearly interpolated.
+     *
+     * Check the individual device manual for notes on the gain range.
+     *
+     * Note that it is not possible to specify a gain name for
+     * this function, it will always set the overall gain.
+     *
+     * \param gain the normalized gain value
+     * \param chan the channel index 0 to N-1
+     * \throws A uhd::runtime_error if the gain value is outside [0, 1].
+     */
+    virtual void set_normalized_rx_gain(double gain, size_t chan = 0) = 0;
+
+    /*!
+     * Enable or disable the RX AGC module.
+     * Once this module is enabled manual gain settings will be ignored.
+     * The AGC will start in a default configuration which should be good for most use cases.
+     * Device specific configuration parameters can be found in the property tree.
+     * \param enable Enable or Disable the AGC
+     * \param chan the channel index 0 to N-1
+     */
+    virtual void set_rx_agc(bool enable, size_t chan = 0) = 0;
+
     /*!
      * Get the RX gain value for the specified gain element.
      * For an empty name, sum across all gain elements.
@@ -501,6 +542,18 @@ public:
         return this->get_rx_gain(ALL_GAINS, chan);
     }
 
+    /*!
+     * Return the normalized RX gain value.
+     *
+     * See set_normalized_rx_gain() for a discussion of normalized
+     * gains.
+     *
+     * \param chan the channel index 0 to N-1
+     * \returns The normalized gain (in [0, 1])
+     * \throws A uhd::runtime_error if the gain value is outside [0, 1].
+     */
+    virtual double get_normalized_rx_gain(size_t chan = 0) = 0;
+
     /*!
      * Get the RX gain range for the specified gain element.
      * For an empty name, calculate the overall gain range.
@@ -544,15 +597,6 @@ public:
      */
     virtual std::vector<std::string> get_rx_antennas(size_t chan = 0) = 0;
 
-    /*!
-     * Get the locked status of the LO on the frontend.
-     * \param chan the channel index 0 to N-1
-     * \return true for locked
-     */
-    UHD_DEPRECATED bool get_rx_lo_locked(size_t chan = 0){
-        return this->get_rx_sensor("lo_locked", chan).to_bool();
-    }
-
     /*!
      * Set the RX bandwidth on the frontend.
      * \param bandwidth the bandwidth in Hz
@@ -574,16 +618,6 @@ public:
      */
     virtual meta_range_t get_rx_bandwidth_range(size_t chan = 0) = 0;
 
-    /*!
-     * Read the RSSI value on the RX frontend.
-     * \param chan the channel index 0 to N-1
-     * \return the rssi in dB
-     * \throw exception if RSSI readback not supported
-     */
-    UHD_DEPRECATED double read_rssi(size_t chan = 0){
-        return this->get_rx_sensor("rssi", chan).to_real();
-    }
-
     /*!
      * Get the dboard interface object for the RX frontend.
      * The dboard interface gives access to GPIOs, SPI, I2C, low-speed ADC and DAC.
@@ -631,6 +665,14 @@ public:
      */
     virtual void set_rx_dc_offset(const std::complex<double> &offset, size_t chan = ALL_CHANS) = 0;
 
+    /*!
+     * Enable/disable the automatic IQ imbalance correction.
+     *
+     * \param enb true to enable automatic IQ balance correction
+     * \param chan the channel index 0 to N-1
+     */
+    virtual void set_rx_iq_balance(const bool enb, size_t chan) = 0;
+
     /*!
      * Set the RX frontend IQ imbalance correction.
      * Use this to adjust the magnitude and phase of I and Q.
@@ -744,6 +786,18 @@ public:
         return this->set_tx_gain(gain, ALL_GAINS, chan);
     }
 
+    /*!
+     * Set the normalized TX gain value.
+     *
+     * See set_normalized_rx_gain() for a discussion on normalized
+     * gains.
+     *
+     * \param gain the normalized gain value
+     * \param chan the channel index 0 to N-1
+     * \throws A uhd::runtime_error if the gain value is outside [0, 1].
+     */
+    virtual void set_normalized_tx_gain(double gain, size_t chan = 0) = 0;
+
     /*!
      * Get the TX gain value for the specified gain element.
      * For an empty name, sum across all gain elements.
@@ -758,6 +812,18 @@ public:
         return this->get_tx_gain(ALL_GAINS, chan);
     }
 
+    /*!
+     * Return the normalized TX gain value.
+     *
+     * See set_normalized_rx_gain() for a discussion of normalized
+     * gains.
+     *
+     * \param chan the channel index 0 to N-1
+     * \returns The normalized gain (in [0, 1])
+     * \throws A uhd::runtime_error if the gain value is outside [0, 1].
+     */
+    virtual double get_normalized_tx_gain(size_t chan = 0) = 0;
+
     /*!
      * Get the TX gain range for the specified gain element.
      * For an empty name, calculate the overall gain range.
@@ -801,15 +867,6 @@ public:
      */
     virtual std::vector<std::string> get_tx_antennas(size_t chan = 0) = 0;
 
-    /*!
-     * Get the locked status of the LO on the frontend.
-     * \param chan the channel index 0 to N-1
-     * \return true for locked
-     */
-    UHD_DEPRECATED bool get_tx_lo_locked(size_t chan = 0){
-        return this->get_tx_sensor("lo_locked", chan).to_bool();
-    }
-
     /*!
      * Set the TX bandwidth on the frontend.
      * \param bandwidth the bandwidth in Hz
@@ -919,6 +976,80 @@ public:
      */
     virtual boost::uint32_t get_gpio_attr(const std::string &bank, const std::string &attr, const size_t mboard = 0) = 0;
 
+    /*******************************************************************
+     * Register IO methods
+     ******************************************************************/
+    struct register_info_t {
+        size_t bitwidth;
+        bool readable;
+        bool writable;
+    };
+
+    /*!
+     * Enumerate the full paths of all low-level USRP registers accessible to read/write
+     * \param mboard the motherboard index 0 to M-1
+     * \return a vector of register paths
+     */
+    virtual std::vector<std::string> enumerate_registers(const size_t mboard = 0) = 0;
+
+    /*!
+     * Get more information about a low-level device register
+     * \param path the full path to the register
+     * \param mboard the motherboard index 0 to M-1
+     * \return the info struct which contains the bitwidth and read-write access information
+     */
+    virtual register_info_t get_register_info(const std::string &path, const size_t mboard = 0) = 0;
+
+    /*!
+     * Write a low-level register field for a register in the USRP hardware
+     * \param path the full path to the register
+     * \param field the identifier of bitfield to be written (all other bits remain unchanged)
+     * \param value the value to write to the register field
+     * \param mboard the motherboard index 0 to M-1
+     */
+    virtual void write_register(const std::string &path, const boost::uint32_t field, const boost::uint64_t value, const size_t mboard = 0) = 0;
+
+    /*!
+     * Read a low-level register field from a register in the USRP hardware
+     * \param path the full path to the register
+     * \param field the identifier of bitfield to be read
+     * \param mboard the motherboard index 0 to M-1
+     * \return the value of the register field
+     */
+    virtual boost::uint64_t read_register(const std::string &path, const boost::uint32_t field, const size_t mboard = 0) = 0;
+
+    /*******************************************************************
+     * Filter API methods
+     ******************************************************************/
+
+    /*!
+     * Enumerate the available filters in the signal path.
+     * \param search_mask
+     * \parblock
+     * Select only certain filter names by specifying this search mask.
+     *
+     * E.g. if search mask is set to "rx_frontends/A" only filter names including that string will be returned.
+     * \endparblock
+     * \return a vector of strings representing the selected filter names.
+     */
+    virtual std::vector<std::string> get_filter_names(const std::string &search_mask = "") = 0;
+
+    /*!
+     * Return the filter object for the given name.
+     * \param path the name of the filter as returned from get_filter_names().
+     * \return a filter_info_base::sptr.
+     */
+    virtual filter_info_base::sptr get_filter(const std::string &path) = 0;
+
+    /*!
+     * Write back a filter obtained by get_filter() to the signal path.
+     * This filter can be a modified version of the originally returned one.
+     * The information about Rx or Tx is contained in the path parameter.
+     * \param path the name of the filter as returned from get_filter_names().
+     * \param filter the filter_info_base::sptr of the filter object to be written
+     */
+    virtual void set_filter(const std::string &path, filter_info_base::sptr filter) = 0;
+
 };
 
 }}
