@@ -1,5 +1,5 @@
 //
-// Copyright 2011 Ettus Research LLC
+// Copyright 2011,2014 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,9 +19,10 @@
 #define INCLUDED_LIBUHD_USRP_RX_FRONTEND_CORE_200_HPP
 
 #include <uhd/config.hpp>
+#include <uhd/types/wb_iface.hpp>
+#include <uhd/property_tree.hpp>
 #include <boost/utility.hpp>
 #include <boost/shared_ptr.hpp>
-#include <uhd/types/wb_iface.hpp>
 #include <complex>
 #include <string>
 
@@ -29,6 +30,11 @@ class tx_frontend_core_200 : boost::noncopyable{
 public:
     typedef boost::shared_ptr<tx_frontend_core_200> sptr;
 
+    static const std::complex<double> DEFAULT_DC_OFFSET_VALUE;
+    static const std::complex<double> DEFAULT_IQ_BALANCE_VALUE;
+
+    virtual ~tx_frontend_core_200(void) = 0;
+
     static sptr make(uhd::wb_iface::sptr iface, const size_t base);
 
     virtual void set_mux(const std::string &mode) = 0;
@@ -37,6 +43,8 @@ public:
 
     virtual void set_iq_balance(const std::complex<double> &cor) = 0;
 
+    virtual void populate_subtree(uhd::property_tree::sptr subtree) = 0;
+
 };
 
 #endif /* INCLUDED_LIBUHD_USRP_RX_FRONTEND_CORE_200_HPP */
