@@ -1,5 +1,5 @@
 //
-// Copyright 2010-2012,2014 Ettus Research LLC
+// Copyright 2010-2012,2014-2015 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,13 +21,14 @@
 #include <uhd/utils/msg.hpp>
 #include <uhd/exception.hpp>
 #include <uhd/utils/static.hpp>
-#include <uhd/utils/images.hpp>
+#include <uhd/utils/paths.hpp>
 #include <boost/bind.hpp>
 #include <boost/format.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/functional/hash.hpp>
 #include <boost/assign/list_of.hpp>
 #include <fstream>
+#include <iostream>
 #include <ctime>
 
 using namespace uhd;
@@ -45,7 +46,7 @@ namespace fs = boost::filesystem;
 /***********************************************************************
  * Discovery
  **********************************************************************/
-static device_addrs_t e100_find(const device_addr_t &hint){
+device_addrs_t e100_find(const device_addr_t &hint){
     device_addrs_t e100_addrs;
 
     //return an empty list of addresses when type is set to non-usrp-e
@@ -96,7 +97,7 @@ static device::sptr e100_make(const device_addr_t &device_addr){
 }
 
 UHD_STATIC_BLOCK(register_e100_device){
-    device::register_device(&e100_find, &e100_make);
+    device::register_device(&e100_find, &e100_make, device::USRP);
 }
 
 static const uhd::dict<std::string, std::string> model_to_fpga_file_name = boost::assign::map_list_of
@@ -104,15 +105,10 @@ static const uhd::dict<std::string, std::string> model_to_fpga_file_name = boost
     ("E110", "usrp_e110_fpga.bin")
 ;
 
-/***********************************************************************
- * Structors
- **********************************************************************/
-e100_impl::e100_impl(const uhd::device_addr_t &device_addr){
-    _tree = property_tree::make();
-
+std::string get_default_e1x0_fpga_image(const uhd::device_addr_t &device_addr){
     //read the eeprom so we can determine the hardware
-    _dev_i2c_iface = e100_ctrl::make_dev_i2c_iface(E100_I2C_DEV_NODE);
-    const mboard_eeprom_t mb_eeprom(*_dev_i2c_iface, E100_EEPROM_MAP_KEY);
+    uhd::i2c_iface::sptr dev_i2c_iface = e100_ctrl::make_dev_i2c_iface(E100_I2C_DEV_NODE);
+    const mboard_eeprom_t mb_eeprom(*dev_i2c_iface, E100_EEPROM_MAP_KEY);
 
     //determine the model string for this device
     const std::string model = device_addr.get("model", mb_eeprom.get("model", ""));
@@ -124,13 +120,27 @@ e100_impl::e100_impl(const uhd::device_addr_t &device_addr){
     ) % model));
 
     //extract the fpga path and compute hash
-    const std::string default_fpga_file_name = model_to_fpga_file_name[model];
+    return model_to_fpga_file_name[model];
+}
+
+/***********************************************************************
+ * Structors
+ **********************************************************************/
+e100_impl::e100_impl(const uhd::device_addr_t &device_addr){
+    _tree = property_tree::make();
+    _type = device::USRP;
+    _ignore_cal_file = device_addr.has_key("ignore-cal-file");
+
+    _dev_i2c_iface = e100_ctrl::make_dev_i2c_iface(E100_I2C_DEV_NODE);
+    const mboard_eeprom_t mb_eeprom(*_dev_i2c_iface, E100_EEPROM_MAP_KEY);
+    const std::string default_fpga_file_name = get_default_e1x0_fpga_image(device_addr);
+    const std::string model = device_addr["model"];
     std::string e100_fpga_image;
     try{
         e100_fpga_image = find_image_path(device_addr.get("fpga", default_fpga_file_name));
     }
     catch(...){
-        UHD_MSG(error) << boost::format("Could not find FPGA image. %s\n") % print_images_error();
+        UHD_MSG(error) << boost::format("Could not find FPGA image. %s\n") % print_utility_error("uhd_images_downloader.py");
         throw;
     }
     e100_load_fpga(e100_fpga_image);
@@ -546,9 +556,13 @@ void e100_impl::check_fpga_compat(void){
 }
 
 void e100_impl::set_rx_fe_corrections(const double lo_freq){
-    apply_rx_fe_corrections(this->get_tree()->subtree("/mboards/0"), "A", lo_freq);
+    if(not _ignore_cal_file){
+        apply_rx_fe_corrections(this->get_tree()->subtree("/mboards/0"), "A", lo_freq);
+    }
 }
 
 void e100_impl::set_tx_fe_corrections(const double lo_freq){
-    apply_tx_fe_corrections(this->get_tree()->subtree("/mboards/0"), "A", lo_freq);
+    if(not _ignore_cal_file){
+        apply_tx_fe_corrections(this->get_tree()->subtree("/mboards/0"), "A", lo_freq);
+    }
 }
