@@ -20,15 +20,19 @@
 #include <uhd/utils/msg.hpp>
 #include <uhd/exception.hpp>
 #include <uhd/utils/log.hpp>
+#include <uhd/utils/math.hpp>
 #include <uhd/utils/gain_group.hpp>
 #include <uhd/usrp/dboard_id.hpp>
 #include <uhd/usrp/mboard_eeprom.hpp>
 #include <uhd/usrp/dboard_eeprom.hpp>
 #include <uhd/convert.hpp>
+#include <uhd/utils/soft_register.hpp>
 #include <boost/assign/list_of.hpp>
 #include <boost/thread.hpp>
 #include <boost/foreach.hpp>
 #include <boost/format.hpp>
+#include <boost/algorithm/string.hpp>
+#include <algorithm>
 #include <cmath>
 
 using namespace uhd;
@@ -68,26 +72,116 @@ static void do_samp_rate_warning_message(
     }
 }
 
-static void do_tune_freq_warning_message(
+/*static void do_tune_freq_results_message(
     const tune_request_t &tune_req,
+    const tune_result_t &tune_result,
     double actual_freq,
     const std::string &xx
 ){
-    //forget the warning when manual policy
-    if (tune_req.dsp_freq_policy == tune_request_t::POLICY_MANUAL) return;
-    if (tune_req.rf_freq_policy == tune_request_t::POLICY_MANUAL) return;
-
     const double target_freq = tune_req.target_freq;
-    static const double max_allowed_error = 1.0; //Hz
-    if (std::abs(target_freq - actual_freq) > max_allowed_error){
-        UHD_MSG(warning) << boost::format(
-            "The hardware does not support the requested %s frequency:\n"
-            "Target frequency: %f MHz\n"
-            "Actual frequency: %f MHz\n"
-        ) % xx % (target_freq/1e6) % (actual_freq/1e6);
-    }
-}
+    const double clipped_target_freq = tune_result.clipped_rf_freq;
+    const double target_rf_freq = tune_result.target_rf_freq;
+    const double actual_rf_freq = tune_result.actual_rf_freq;
+    const double target_dsp_freq = tune_result.target_dsp_freq;
+    const double actual_dsp_freq = tune_result.actual_dsp_freq;
 
+    if (tune_req.rf_freq_policy == tune_request_t::POLICY_MANUAL) return;
+    if (tune_req.dsp_freq_policy == tune_request_t::POLICY_MANUAL) return;
+
+    bool requested_freq_success = uhd::math::frequencies_are_equal(target_freq, clipped_target_freq);
+    bool target_freq_success = uhd::math::frequencies_are_equal(clipped_target_freq, actual_freq);
+    bool rf_lo_tune_success = uhd::math::frequencies_are_equal(target_rf_freq, actual_rf_freq);
+    bool dsp_tune_success = uhd::math::frequencies_are_equal(target_dsp_freq, actual_dsp_freq);
+
+    if(requested_freq_success and target_freq_success and rf_lo_tune_success
+            and dsp_tune_success) {
+        UHD_MSG(status) << boost::format(
+                "Successfully tuned to %f MHz\n\n")
+                % (actual_freq / 1e6);
+    } else {
+        boost::format base_message ("Tune Request: %f MHz\n");
+        base_message % (target_freq / 1e6);
+        std::string results_string = base_message.str();
+
+        if(requested_freq_success and (not rf_lo_tune_success)) {
+            boost::format rf_lo_message(
+                "  The RF LO does not support the requested frequency:\n"
+                "    Requested LO Frequency: %f MHz\n"
+                "    RF LO Result: %f MHz\n"
+                "  Attempted to use the DSP to reach the requested frequency:\n"
+                "    Desired DSP Frequency: %f MHz\n"
+                "    DSP Result: %f MHz\n"
+                "  Successfully tuned to %f MHz\n\n");
+            rf_lo_message % (target_rf_freq / 1e6) % (actual_rf_freq / 1e6)
+                % (target_dsp_freq / 1e6) % (actual_dsp_freq / 1e6)
+                % (actual_freq / 1e6);
+
+            results_string += rf_lo_message.str();
+
+            UHD_MSG(status) << results_string;
+
+            return;
+        }
+
+        if(not requested_freq_success) {
+            boost::format failure_message(
+                "  The requested %s frequency is outside of the system range, and has been clipped:\n"
+                "    Target Frequency: %f MHz\n"
+                "    Clipped Target Frequency: %f MHz\n");
+            failure_message % xx % (target_freq / 1e6) % (clipped_target_freq / 1e6);
+
+            results_string += failure_message.str();
+        }
+
+        if(not rf_lo_tune_success) {
+            boost::format rf_lo_message(
+                "  The RF LO does not support the requested frequency:\n"
+                "    Requested LO Frequency: %f MHz\n"
+                "    RF LO Result: %f MHz\n"
+                "  Attempted to use the DSP to reach the requested frequency:\n"
+                "    Desired DSP Frequency: %f MHz\n"
+                "    DSP Result: %f MHz\n");
+            rf_lo_message % (target_rf_freq / 1e6) % (actual_rf_freq / 1e6)
+                % (target_dsp_freq / 1e6) % (actual_dsp_freq / 1e6);
+
+            results_string += rf_lo_message.str();
+
+        } else if(not dsp_tune_success) {
+            boost::format dsp_message(
+                "  The DSP does not support the requested frequency:\n"
+                "    Requested DSP Frequency: %f MHz\n"
+                "    DSP Result: %f MHz\n");
+            dsp_message % (target_dsp_freq / 1e6) % (actual_dsp_freq / 1e6);
+
+            results_string += dsp_message.str();
+        }
+
+        if(target_freq_success) {
+            boost::format success_message(
+                "  Successfully tuned to %f MHz\n\n");
+            success_message % (actual_freq / 1e6);
+
+            results_string += success_message.str();
+        } else {
+            boost::format failure_message(
+                "  Failed to tune to target frequency\n"
+                "    Target Frequency: %f MHz\n"
+                "    Actual Frequency: %f MHz\n\n");
+            failure_message % (clipped_target_freq / 1e6) % (actual_freq / 1e6);
+
+            results_string += failure_message.str();
+        }
+
+        UHD_MSG(warning) << results_string << std::endl;
+    }
+}*/
+
+/*! The CORDIC can be used to shift the baseband below / past the tunable
+ * limits of the actual RF front-end. The baseband filter, located on the
+ * daughterboard, however, limits the useful instantaneous bandwidth. We
+ * allow the user to tune to the edge of the filter, where the roll-off
+ * begins.  This prevents the user from tuning past the point where less
+ * than half of the spectrum would be useful. */
 static meta_range_t make_overall_tune_range(
     const meta_range_t &fe_range,
     const meta_range_t &dsp_range,
@@ -142,12 +236,36 @@ static tune_result_t tune_xx_subdev_and_dsp(
     const tune_request_t &tune_request
 ){
     //------------------------------------------------------------------
-    //-- calculate the LO offset, only used with automatic policy
+    //-- calculate the tunable frequency ranges of the system
     //------------------------------------------------------------------
+    freq_range_t tune_range = make_overall_tune_range(
+            rf_fe_subtree->access<meta_range_t>("freq/range").get(),
+            dsp_subtree->access<meta_range_t>("freq/range").get(),
+            rf_fe_subtree->access<double>("bandwidth/value").get()
+        );
+
+    freq_range_t dsp_range = dsp_subtree->access<meta_range_t>("freq/range").get();
+    freq_range_t rf_range = rf_fe_subtree->access<meta_range_t>("freq/range").get();
+
+    double clipped_requested_freq = tune_range.clip(tune_request.target_freq);
+
+    //------------------------------------------------------------------
+    //-- If the RF FE requires an LO offset, build it into the tune request
+    //------------------------------------------------------------------
+
+    /*! The automatically calculated LO offset is only used if the
+     * 'use_lo_offset' field in the daughterboard property tree is set to TRUE,
+     * and the tune policy is set to AUTO. To use an LO offset normally, the
+     * user should specify the MANUAL tune policy and lo_offset as part of the
+     * tune_request. This lo_offset is based on the requirements of the FE, and
+     * does not reflect a user-requested lo_offset, which is handled later. */
     double lo_offset = 0.0;
     if (rf_fe_subtree->access<bool>("use_lo_offset").get()){
-        //If the frontend has lo_offset value and range properties, trust it for lo_offset
-        if (rf_fe_subtree->exists("lo_offset/value")) lo_offset = rf_fe_subtree->access<double>("lo_offset/value").get();
+        // If the frontend has lo_offset value and range properties, trust it
+        // for lo_offset
+        if (rf_fe_subtree->exists("lo_offset/value")) {
+            lo_offset = rf_fe_subtree->access<double>("lo_offset/value").get();
+        }
 
         //If the local oscillator will be in the passband, use an offset.
         //But constrain the LO offset by the width of the filter bandwidth.
@@ -159,8 +277,7 @@ static tune_result_t tune_xx_subdev_and_dsp(
     //------------------------------------------------------------------
     //-- poke the tune request args into the dboard
     //------------------------------------------------------------------
-    if (rf_fe_subtree->exists("tune_args"))
-    {
+    if (rf_fe_subtree->exists("tune_args")) {
         rf_fe_subtree->access<device_addr_t>("tune_args").set(tune_request.args);
     }
 
@@ -168,58 +285,82 @@ static tune_result_t tune_xx_subdev_and_dsp(
     //-- set the RF frequency depending upon the policy
     //------------------------------------------------------------------
     double target_rf_freq = 0.0;
+
     switch (tune_request.rf_freq_policy){
-    case tune_request_t::POLICY_AUTO:
-        target_rf_freq = tune_request.target_freq + lo_offset;
+        case tune_request_t::POLICY_AUTO:
+            target_rf_freq = clipped_requested_freq + lo_offset;
+            break;
+
+        case tune_request_t::POLICY_MANUAL:
+            // If the rf_fe understands lo_offset settings, infer the desired
+            // lo_offset and set it. Side effect: In TVRX2 for example, after
+            // setting the lo_offset (if_freq) with a POLICY_MANUAL, there is no
+            // way for the user to automatically get back to default if_freq
+            // without deconstruct/reconstruct the rf_fe objects.
+            if (rf_fe_subtree->exists("lo_offset/value")) {
+                rf_fe_subtree->access<double>("lo_offset/value")
+                    .set(tune_request.rf_freq - tune_request.target_freq);
+            }
+
+            target_rf_freq = rf_range.clip(tune_request.rf_freq);
+            break;
+
+        case tune_request_t::POLICY_NONE:
+            break; //does not set
+    }
+
+    //------------------------------------------------------------------
+    //-- Tune the RF frontend
+    //------------------------------------------------------------------
+    if (tune_request.rf_freq_policy != tune_request_t::POLICY_NONE) {
         rf_fe_subtree->access<double>("freq/value").set(target_rf_freq);
-        break;
-
-    case tune_request_t::POLICY_MANUAL:
-        //If the rf_fe understands lo_offset settings, infer the desired lo_offset and set it
-        //  Side effect: In TVRX2 for example, after setting the lo_offset (if_freq) with a
-        //  POLICY_MANUAL, there is no way for the user to automatically get back to default
-        //  if_freq without deconstruct/reconstruct the rf_fe objects.
-        if (rf_fe_subtree->exists("lo_offset/value")) {
-            rf_fe_subtree->access<double>("lo_offset/value").set(tune_request.rf_freq - tune_request.target_freq);
-        }
-
-        target_rf_freq = tune_request.rf_freq;
-        rf_fe_subtree->access<double>("freq/value").set(target_rf_freq);
-        break;
-
-    case tune_request_t::POLICY_NONE: break; //does not set
     }
     const double actual_rf_freq = rf_fe_subtree->access<double>("freq/value").get();
 
     //------------------------------------------------------------------
-    //-- calculate the dsp freq, only used with automatic policy
+    //-- Set the DSP frequency depending upon the DSP frequency policy.
     //------------------------------------------------------------------
-    double target_dsp_freq = actual_rf_freq - tune_request.target_freq;
+    double target_dsp_freq = 0.0;
+    switch (tune_request.dsp_freq_policy) {
+        case tune_request_t::POLICY_AUTO:
+            /* If we are using the AUTO tuning policy, then we prevent the
+             * CORDIC from spinning us outside of the range of the baseband
+             * filter, regardless of what the user requested. This could happen
+             * if the user requested a center frequency so far outside of the
+             * tunable range of the FE that the CORDIC would spin outside the
+             * filtered baseband. */
+            target_dsp_freq = actual_rf_freq - clipped_requested_freq;
 
-    //invert the sign on the dsp freq for transmit
-    target_dsp_freq *= xx_sign;
+            //invert the sign on the dsp freq for transmit (spinning up vs down)
+            target_dsp_freq *= xx_sign;
+
+            break;
+
+        case tune_request_t::POLICY_MANUAL:
+            /* If the user has specified a manual tune policy, we will allow
+             * tuning outside of the baseband filter, but will still clip the
+             * target DSP frequency to within the bounds of the CORDIC to
+             * prevent undefined behavior (likely an overflow). */
+            target_dsp_freq = dsp_range.clip(tune_request.dsp_freq);
+            break;
+
+        case tune_request_t::POLICY_NONE:
+            break; //does not set
+    }
 
     //------------------------------------------------------------------
-    //-- set the dsp frequency depending upon the dsp frequency policy
+    //-- Tune the DSP
     //------------------------------------------------------------------
-    switch (tune_request.dsp_freq_policy){
-    case tune_request_t::POLICY_AUTO:
+    if (tune_request.dsp_freq_policy != tune_request_t::POLICY_NONE) {
         dsp_subtree->access<double>("freq/value").set(target_dsp_freq);
-        break;
-
-    case tune_request_t::POLICY_MANUAL:
-        target_dsp_freq = tune_request.dsp_freq;
-        dsp_subtree->access<double>("freq/value").set(target_dsp_freq);
-        break;
-
-    case tune_request_t::POLICY_NONE: break; //does not set
     }
     const double actual_dsp_freq = dsp_subtree->access<double>("freq/value").get();
 
     //------------------------------------------------------------------
-    //-- load and return the tune result
+    //-- Load and return the tune result
     //------------------------------------------------------------------
     tune_result_t tune_result;
+    tune_result.clipped_rf_freq = clipped_requested_freq;
     tune_result.target_rf_freq = target_rf_freq;
     tune_result.actual_rf_freq = actual_rf_freq;
     tune_result.target_dsp_freq = target_dsp_freq;
@@ -246,7 +387,7 @@ static double derive_freq_from_xx_subdev_and_dsp(
 class multi_usrp_impl : public multi_usrp{
 public:
     multi_usrp_impl(const device_addr_t &addr){
-        _dev = device::make(addr);
+        _dev = device::make(addr, device::USRP);
         _tree = _dev->get_tree();
     }
 
@@ -259,16 +400,16 @@ public:
         dict<std::string, std::string> usrp_info;
 
         mboard_eeprom_t mb_eeprom = _tree->access<mboard_eeprom_t>(mb_root(mcp.mboard) / "eeprom").get();
-        dboard_eeprom_t db_eeprom = _tree->access<dboard_eeprom_t>(rx_rf_fe_root(mcp.chan).branch_path().branch_path() / "rx_eeprom").get();
+        dboard_eeprom_t db_eeprom = _tree->access<dboard_eeprom_t>(rx_rf_fe_root(chan).branch_path().branch_path() / "rx_eeprom").get();
 
         usrp_info["mboard_id"] = _tree->access<std::string>(mb_root(mcp.mboard) / "name").get();
         usrp_info["mboard_name"] = mb_eeprom["name"];
         usrp_info["mboard_serial"] = mb_eeprom["serial"];
         usrp_info["rx_id"] = db_eeprom.id.to_pp_string();
-        usrp_info["rx_subdev_name"] = _tree->access<std::string>(rx_rf_fe_root(mcp.chan) / "name").get();
+        usrp_info["rx_subdev_name"] = _tree->access<std::string>(rx_rf_fe_root(chan) / "name").get();
         usrp_info["rx_subdev_spec"] = _tree->access<subdev_spec_t>(mb_root(mcp.mboard) / "rx_subdev_spec").get().to_string();
         usrp_info["rx_serial"] = db_eeprom.serial;
-        usrp_info["rx_antenna"] =  _tree->access<std::string>(rx_rf_fe_root(mcp.chan) / "antenna" / "value").get();
+        usrp_info["rx_antenna"] =  _tree->access<std::string>(rx_rf_fe_root(chan) / "antenna" / "value").get();
 
         return usrp_info;
     }
@@ -278,16 +419,16 @@ public:
         dict<std::string, std::string> usrp_info;
 
         mboard_eeprom_t mb_eeprom = _tree->access<mboard_eeprom_t>(mb_root(mcp.mboard) / "eeprom").get();
-        dboard_eeprom_t db_eeprom = _tree->access<dboard_eeprom_t>(tx_rf_fe_root(mcp.chan).branch_path().branch_path() / "tx_eeprom").get();
+        dboard_eeprom_t db_eeprom = _tree->access<dboard_eeprom_t>(tx_rf_fe_root(chan).branch_path().branch_path() / "tx_eeprom").get();
 
         usrp_info["mboard_id"] = _tree->access<std::string>(mb_root(mcp.mboard) / "name").get();
         usrp_info["mboard_name"] = mb_eeprom["name"];
         usrp_info["mboard_serial"] = mb_eeprom["serial"];
         usrp_info["tx_id"] = db_eeprom.id.to_pp_string();
-        usrp_info["tx_subdev_name"] = _tree->access<std::string>(tx_rf_fe_root(mcp.chan) / "name").get();
+        usrp_info["tx_subdev_name"] = _tree->access<std::string>(tx_rf_fe_root(chan) / "name").get();
         usrp_info["tx_subdev_spec"] = _tree->access<subdev_spec_t>(mb_root(mcp.mboard) / "tx_subdev_spec").get().to_string();
         usrp_info["tx_serial"] = db_eeprom.serial;
-        usrp_info["tx_antenna"] = _tree->access<std::string>(tx_rf_fe_root(mcp.chan) / "antenna" / "value").get();
+        usrp_info["tx_antenna"] = _tree->access<std::string>(tx_rf_fe_root(chan) / "antenna" / "value").get();
 
         return usrp_info;
     }
@@ -297,6 +438,9 @@ public:
      ******************************************************************/
     void set_master_clock_rate(double rate, size_t mboard){
         if (mboard != ALL_MBOARDS){
+            if (_tree->exists(mb_root(mboard) / "auto_tick_rate")) {
+                _tree->access<bool>(mb_root(mboard) / "auto_tick_rate").set(false);
+            }
             _tree->access<double>(mb_root(mboard) / "tick_rate").set(rate);
             return;
         }
@@ -662,9 +806,12 @@ public:
     }
 
     tune_result_t set_rx_freq(const tune_request_t &tune_request, size_t chan){
-        tune_result_t r = tune_xx_subdev_and_dsp(RX_SIGN, _tree->subtree(rx_dsp_root(chan)), _tree->subtree(rx_rf_fe_root(chan)), tune_request);
-        do_tune_freq_warning_message(tune_request, get_rx_freq(chan), "RX");
-        return r;
+        tune_result_t result = tune_xx_subdev_and_dsp(RX_SIGN,
+                _tree->subtree(rx_dsp_root(chan)),
+                _tree->subtree(rx_rf_fe_root(chan)),
+                tune_request);
+        //do_tune_freq_results_message(tune_request, result, get_rx_freq(chan), "RX");
+        return result;
     }
 
     double get_rx_freq(size_t chan){
@@ -684,25 +831,86 @@ public:
     }
 
     void set_rx_gain(double gain, const std::string &name, size_t chan){
+        /* Check if any AGC mode is enable and if so warn the user */
+        if (chan != ALL_CHANS) {
+            if (_tree->exists(rx_rf_fe_root(chan) / "gain" / "agc")) {
+                bool agc = _tree->access<bool>(rx_rf_fe_root(chan) / "gain" / "agc" / "enable").get();
+                if(agc) {
+                    UHD_MSG(warning) << "AGC enabled for this channel. Setting will be ignored." << std::endl;
+                }
+            }
+        } else {
+            for (size_t c = 0; c < get_rx_num_channels(); c++){
+                if (_tree->exists(rx_rf_fe_root(c) / "gain" / "agc")) {
+                    bool agc = _tree->access<bool>(rx_rf_fe_root(chan) / "gain" / "agc" / "enable").get();
+                    if(agc) {
+                        UHD_MSG(warning) << "AGC enabled for this channel. Setting will be ignored." << std::endl;
+                    }
+                }
+            }
+        }
+        /* Apply gain setting.
+         * If device is in AGC mode it will ignore the setting. */
         try {
             return rx_gain_group(chan)->set_value(gain, name);
-        } catch (uhd::key_error &e) {
+        } catch (uhd::key_error &) {
             THROW_GAIN_NAME_ERROR(name,chan,rx);
         }
     }
 
+    void set_normalized_rx_gain(double gain, size_t chan = 0)
+    {
+      if (gain > 1.0 || gain < 0.0) {
+        throw uhd::runtime_error("Normalized gain out of range, must be in [0, 1].");
+      }
+      gain_range_t gain_range = get_rx_gain_range(ALL_GAINS, chan);
+      double abs_gain = (gain * (gain_range.stop() - gain_range.start())) + gain_range.start();
+      set_rx_gain(abs_gain, ALL_GAINS, chan);
+    }
+
+    void set_rx_agc(bool enable, size_t chan = 0)
+    {
+        if (chan != ALL_CHANS){
+            if (_tree->exists(rx_rf_fe_root(chan) / "gain" / "agc" / "enable")) {
+                _tree->access<bool>(rx_rf_fe_root(chan) / "gain" / "agc" / "enable").set(enable);
+            } else {
+                UHD_MSG(warning) << "AGC is not available on this device." << std::endl;
+            }
+            return;
+        }
+        for (size_t c = 0; c < get_rx_num_channels(); c++){
+            this->set_rx_agc(enable, c);
+        }
+
+    }
+
     double get_rx_gain(const std::string &name, size_t chan){
         try {
             return rx_gain_group(chan)->get_value(name);
-        } catch (uhd::key_error &e) {
+        } catch (uhd::key_error &) {
             THROW_GAIN_NAME_ERROR(name,chan,rx);
         }
     }
 
+    double get_normalized_rx_gain(size_t chan)
+    {
+      gain_range_t gain_range = get_rx_gain_range(ALL_GAINS, chan);
+      double gain_range_width = gain_range.stop() - gain_range.start();
+      // In case we have a device without a range of gains:
+      if (gain_range_width == 0.0) {
+          return 0;
+      }
+      double norm_gain = (get_rx_gain(ALL_GAINS, chan) - gain_range.start()) / gain_range_width;
+      // Avoid rounding errors:
+      if (norm_gain > 1.0) return 1.0;
+      if (norm_gain < 0.0) return 0.0;
+      return norm_gain;
+    }
+
     gain_range_t get_rx_gain_range(const std::string &name, size_t chan){
         try {
             return rx_gain_group(chan)->get_range(name);
-        } catch (uhd::key_error &e) {
+        } catch (uhd::key_error &) {
             THROW_GAIN_NAME_ERROR(name,chan,rx);
         }
     }
@@ -751,6 +959,9 @@ public:
         if (chan != ALL_CHANS){
             if (_tree->exists(rx_fe_root(chan) / "dc_offset" / "enable")) {
                 _tree->access<bool>(rx_fe_root(chan) / "dc_offset" / "enable").set(enb);
+            } else if (_tree->exists(rx_rf_fe_root(chan) / "dc_offset" / "enable")) {
+                /*For B2xx devices the dc-offset correction is implemented in the rf front-end*/
+                _tree->access<bool>(rx_rf_fe_root(chan) / "dc_offset" / "enable").set(enb);
             } else {
                 UHD_MSG(warning) << "Setting DC offset compensation is not possible on this device." << std::endl;
             }
@@ -775,6 +986,20 @@ public:
         }
     }
 
+    void set_rx_iq_balance(const bool enb, size_t chan){
+        if (chan != ALL_CHANS){
+            if (_tree->exists(rx_rf_fe_root(chan) / "iq_balance" / "enable")) {
+                _tree->access<bool>(rx_rf_fe_root(chan) / "iq_balance" / "enable").set(enb);
+            } else {
+                UHD_MSG(warning) << "Setting IQ imbalance compensation is not possible on this device." << std::endl;
+            }
+            return;
+        }
+        for (size_t c = 0; c < get_rx_num_channels(); c++){
+            this->set_rx_iq_balance(enb, c);
+        }
+    }
+
     void set_rx_iq_balance(const std::complex<double> &offset, size_t chan){
         if (chan != ALL_CHANS){
             if (_tree->exists(rx_fe_root(chan) / "iq_balance" / "value")) {
@@ -789,6 +1014,87 @@ public:
         }
     }
 
+    std::vector<std::string> get_filter_names(const std::string &search_mask)
+    {
+        std::vector<std::string> ret;
+
+        for (size_t chan = 0; chan < get_rx_num_channels(); chan++){
+
+            if (_tree->exists(rx_rf_fe_root(chan) / "filters")) {
+                std::vector<std::string> names = _tree->list(rx_rf_fe_root(chan) / "filters");
+                for(size_t i = 0; i < names.size(); i++)
+                {
+                    std::string name = rx_rf_fe_root(chan) / "filters" / names[i];
+                    if((search_mask.empty()) or boost::contains(name, search_mask)) {
+                        ret.push_back(name);
+                    }
+                }
+            }
+            if (_tree->exists(rx_dsp_root(chan) / "filters")) {
+                std::vector<std::string> names = _tree->list(rx_dsp_root(chan) / "filters");
+                for(size_t i = 0; i < names.size(); i++)
+                {
+                    std::string name = rx_dsp_root(chan) / "filters" / names[i];
+                    if((search_mask.empty()) or (boost::contains(name, search_mask))) {
+                        ret.push_back(name);
+                    }
+                }
+            }
+
+        }
+
+        for (size_t chan = 0; chan < get_tx_num_channels(); chan++){
+
+            if (_tree->exists(tx_rf_fe_root(chan) / "filters")) {
+                std::vector<std::string> names = _tree->list(tx_rf_fe_root(chan) / "filters");
+                for(size_t i = 0; i < names.size(); i++)
+                {
+                    std::string name = tx_rf_fe_root(chan) / "filters" / names[i];
+                    if((search_mask.empty()) or (boost::contains(name, search_mask))) {
+                        ret.push_back(name);
+                    }
+                }
+            }
+            if (_tree->exists(rx_dsp_root(chan) / "filters")) {
+                std::vector<std::string> names = _tree->list(tx_dsp_root(chan) / "filters");
+                for(size_t i = 0; i < names.size(); i++)
+                {
+                    std::string name = tx_dsp_root(chan) / "filters" / names[i];
+                    if((search_mask.empty()) or (boost::contains(name, search_mask))) {
+                        ret.push_back(name);
+                    }
+                }
+            }
+
+        }
+
+        return ret;
+    }
+
+    filter_info_base::sptr get_filter(const std::string &path)
+    {
+        std::vector<std::string> possible_names = get_filter_names("");
+        std::vector<std::string>::iterator it;
+        it = find(possible_names.begin(), possible_names.end(), path);
+        if (it == possible_names.end()) {
+            throw uhd::runtime_error("Attempting to get non-existing filter: "+path);
+        }
+
+        return _tree->access<filter_info_base::sptr>(path / "value").get();
+    }
+
+    void set_filter(const std::string &path, filter_info_base::sptr filter)
+    {
+        std::vector<std::string> possible_names = get_filter_names("");
+        std::vector<std::string>::iterator it;
+        it = find(possible_names.begin(), possible_names.end(), path);
+        if (it == possible_names.end()) {
+            throw uhd::runtime_error("Attempting to set non-existing filter: "+path);
+        }
+
+        _tree->access<filter_info_base::sptr>(path / "value").set(filter);
+    }
+
     /*******************************************************************
      * TX methods
      ******************************************************************/
@@ -860,9 +1166,12 @@ public:
     }
 
     tune_result_t set_tx_freq(const tune_request_t &tune_request, size_t chan){
-        tune_result_t r = tune_xx_subdev_and_dsp(TX_SIGN, _tree->subtree(tx_dsp_root(chan)), _tree->subtree(tx_rf_fe_root(chan)), tune_request);
-        do_tune_freq_warning_message(tune_request, get_tx_freq(chan), "TX");
-        return r;
+        tune_result_t result = tune_xx_subdev_and_dsp(TX_SIGN,
+                _tree->subtree(tx_dsp_root(chan)),
+                _tree->subtree(tx_rf_fe_root(chan)),
+                tune_request);
+        //do_tune_freq_results_message(tune_request, result, get_tx_freq(chan), "TX");
+        return result;
     }
 
     double get_tx_freq(size_t chan){
@@ -884,23 +1193,49 @@ public:
     void set_tx_gain(double gain, const std::string &name, size_t chan){
         try {
             return tx_gain_group(chan)->set_value(gain, name);
-        } catch (uhd::key_error &e) {
+        } catch (uhd::key_error &) {
             THROW_GAIN_NAME_ERROR(name,chan,tx);
         }
     }
 
+    void set_normalized_tx_gain(double gain, size_t chan = 0)
+    {
+      if (gain > 1.0 || gain < 0.0) {
+        throw uhd::runtime_error("Normalized gain out of range, must be in [0, 1].");
+      }
+      gain_range_t gain_range = get_tx_gain_range(ALL_GAINS, chan);
+      double abs_gain = (gain * (gain_range.stop() - gain_range.start())) + gain_range.start();
+      set_tx_gain(abs_gain, ALL_GAINS, chan);
+    }
+
+
     double get_tx_gain(const std::string &name, size_t chan){
         try {
             return tx_gain_group(chan)->get_value(name);
-        } catch (uhd::key_error &e) {
+        } catch (uhd::key_error &) {
             THROW_GAIN_NAME_ERROR(name,chan,tx);
         }
     }
 
+    double get_normalized_tx_gain(size_t chan)
+    {
+      gain_range_t gain_range = get_tx_gain_range(ALL_GAINS, chan);
+      double gain_range_width = gain_range.stop() - gain_range.start();
+      // In case we have a device without a range of gains:
+      if (gain_range_width == 0.0) {
+          return 0.0;
+      }
+      double norm_gain = (get_rx_gain(ALL_GAINS, chan) - gain_range.start()) / gain_range_width;
+      // Avoid rounding errors:
+      if (norm_gain > 1.0) return 1.0;
+      if (norm_gain < 0.0) return 0.0;
+      return norm_gain;
+    }
+
     gain_range_t get_tx_gain_range(const std::string &name, size_t chan){
         try {
             return tx_gain_group(chan)->get_range(name);
-        } catch (uhd::key_error &e) {
+        } catch (uhd::key_error &) {
             THROW_GAIN_NAME_ERROR(name,chan,tx);
         }
     }
@@ -1022,7 +1357,7 @@ public:
     {
         if (_tree->exists(mb_root(mboard) / "gpio" / bank))
         {
-            return _tree->access<boost::uint64_t>(mb_root(mboard) / "gpio" / bank / attr).get();
+            return boost::uint32_t(_tree->access<boost::uint64_t>(mb_root(mboard) / "gpio" / bank / attr).get());
         }
         if (bank.size() > 2 and bank[1] == 'X')
         {
@@ -1041,6 +1376,121 @@ public:
         return 0;
     }
 
+    void write_register(const std::string &path, const boost::uint32_t field, const boost::uint64_t value, const size_t mboard)
+    {
+        if (_tree->exists(mb_root(mboard) / "registers"))
+        {
+            uhd::soft_regmap_accessor_t::sptr accessor =
+                _tree->access<uhd::soft_regmap_accessor_t::sptr>(mb_root(mboard) / "registers").get();
+            uhd::soft_register_base& reg = accessor->lookup(path);
+
+            if (not reg.is_writable()) {
+                throw uhd::runtime_error("multi_usrp::write_register - register not writable: " + path);
+            }
+
+            switch (reg.get_bitwidth()) {
+            case 16:
+                if (reg.is_readable())
+                    uhd::soft_register_base::cast<uhd::soft_reg16_rw_t>(reg).write(field, static_cast<boost::uint16_t>(value));
+                else
+                    uhd::soft_register_base::cast<uhd::soft_reg16_wo_t>(reg).write(field, static_cast<boost::uint16_t>(value));
+            break;
+
+            case 32:
+                if (reg.is_readable())
+                    uhd::soft_register_base::cast<uhd::soft_reg32_rw_t>(reg).write(field, static_cast<boost::uint32_t>(value));
+                else
+                    uhd::soft_register_base::cast<uhd::soft_reg32_wo_t>(reg).write(field, static_cast<boost::uint32_t>(value));
+            break;
+
+            case 64:
+                if (reg.is_readable())
+                    uhd::soft_register_base::cast<uhd::soft_reg64_rw_t>(reg).write(field, value);
+                else
+                    uhd::soft_register_base::cast<uhd::soft_reg64_wo_t>(reg).write(field, value);
+            break;
+
+            default:
+                throw uhd::assertion_error("multi_usrp::write_register - register has invalid bitwidth");
+            }
+
+        } else {
+            throw uhd::not_implemented_error("multi_usrp::write_register - register IO not supported for this device");
+        }
+    }
+
+    boost::uint64_t read_register(const std::string &path, const boost::uint32_t field, const size_t mboard)
+    {
+        if (_tree->exists(mb_root(mboard) / "registers"))
+        {
+            uhd::soft_regmap_accessor_t::sptr accessor =
+                _tree->access<uhd::soft_regmap_accessor_t::sptr>(mb_root(mboard) / "registers").get();
+            uhd::soft_register_base& reg = accessor->lookup(path);
+
+            if (not reg.is_readable()) {
+                throw uhd::runtime_error("multi_usrp::read_register - register not readable: " + path);
+            }
+
+            switch (reg.get_bitwidth()) {
+            case 16:
+                if (reg.is_writable())
+                    return static_cast<boost::uint64_t>(uhd::soft_register_base::cast<uhd::soft_reg16_rw_t>(reg).read(field));
+                else
+                    return static_cast<boost::uint64_t>(uhd::soft_register_base::cast<uhd::soft_reg16_ro_t>(reg).read(field));
+            break;
+
+            case 32:
+                if (reg.is_writable())
+                    return static_cast<boost::uint64_t>(uhd::soft_register_base::cast<uhd::soft_reg32_rw_t>(reg).read(field));
+                else
+                    return static_cast<boost::uint64_t>(uhd::soft_register_base::cast<uhd::soft_reg32_ro_t>(reg).read(field));
+            break;
+
+            case 64:
+                if (reg.is_writable())
+                    return uhd::soft_register_base::cast<uhd::soft_reg64_rw_t>(reg).read(field);
+                else
+                    return uhd::soft_register_base::cast<uhd::soft_reg64_ro_t>(reg).read(field);
+            break;
+
+            default:
+                throw uhd::assertion_error("multi_usrp::read_register - register has invalid bitwidth: " + path);
+            }
+        } else {
+            throw uhd::not_implemented_error("multi_usrp::read_register - register IO not supported for this device");
+        }
+    }
+
+    std::vector<std::string> enumerate_registers(const size_t mboard)
+    {
+        if (_tree->exists(mb_root(mboard) / "registers"))
+        {
+            uhd::soft_regmap_accessor_t::sptr accessor =
+                _tree->access<uhd::soft_regmap_accessor_t::sptr>(mb_root(mboard) / "registers").get();
+            return accessor->enumerate();
+        } else {
+            return std::vector<std::string>();
+        }
+    }
+
+    register_info_t get_register_info(const std::string &path, const size_t mboard = 0)
+    {
+        if (_tree->exists(mb_root(mboard) / "registers"))
+        {
+            uhd::soft_regmap_accessor_t::sptr accessor =
+                _tree->access<uhd::soft_regmap_accessor_t::sptr>(mb_root(mboard) / "registers").get();
+            uhd::soft_register_base& reg = accessor->lookup(path);
+
+            register_info_t info;
+            info.bitwidth = reg.get_bitwidth();
+            info.readable = reg.is_readable();
+            info.writable = reg.is_writable();
+            return info;
+        } else {
+            throw uhd::not_implemented_error("multi_usrp::read_register - register IO not supported for this device");
+        }
+    }
+
 private:
     device::sptr _dev;
     property_tree::sptr _tree;
@@ -1243,6 +1693,10 @@ private:
     }
 };
 
+multi_usrp::~multi_usrp(void){
+    /* NOP */
+}
+
 /***********************************************************************
  * The Make Function
  **********************************************************************/
