@@ -1,5 +1,5 @@
 //
-// Copyright 2010-2012 Ettus Research LLC
+// Copyright 2010-2012,2014 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include <uhd/utils/cast.hpp>
 #include <uhd/exception.hpp>
 #include <uhd/utils/static.hpp>
-#include <uhd/utils/images.hpp>
+#include <uhd/utils/paths.hpp>
 #include <boost/format.hpp>
 #include <boost/assign/list_of.hpp>
 #include <boost/filesystem.hpp>
@@ -85,7 +85,7 @@ static device_addrs_t usrp1_find(const device_addr_t &hint)
             usrp1_fw_image = find_image_path(hint.get("fw", "usrp1_fw.ihx"));
         }
         catch(...){
-            UHD_MSG(warning) << boost::format("Could not locate USRP1 firmware. %s") % print_images_error();
+            UHD_MSG(warning) << boost::format("Could not locate USRP1 firmware. %s") % print_utility_error("uhd_images_downloader.py");
         }
         UHD_LOG << "USRP1 firmware image: " << usrp1_fw_image << std::endl;
 
@@ -139,7 +139,7 @@ static device::sptr usrp1_make(const device_addr_t &device_addr){
 }
 
 UHD_STATIC_BLOCK(register_usrp1_device){
-    device::register_device(&usrp1_find, &usrp1_make);
+    device::register_device(&usrp1_find, &usrp1_make, device::USRP);
 }
 
 /***********************************************************************
@@ -147,6 +147,7 @@ UHD_STATIC_BLOCK(register_usrp1_device){
  **********************************************************************/
 usrp1_impl::usrp1_impl(const device_addr_t &device_addr){
     UHD_MSG(status) << "Opening a USRP1 device..." << std::endl;
+    _type = device::USRP;
 
     //extract the FPGA path for the USRP1
     std::string usrp1_fpga_image = find_image_path(
