@@ -1,5 +1,5 @@
 //
-// Copyright 2010-2012 Ettus Research LLC
+// Copyright 2010-2012,2014 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ static const size_t DEFAULT_NUM_FRAMES = 32;
 /***********************************************************************
  * Discovery over the udp transport
  **********************************************************************/
-static device_addrs_t usrp2_find(const device_addr_t &hint_){
+device_addrs_t usrp2_find(const device_addr_t &hint_){
     //handle the multi-device discovery
     device_addrs_t hints = separate_device_addr(hint_);
     if (hints.size() > 1){
@@ -200,7 +200,7 @@ static device::sptr usrp2_make(const device_addr_t &device_addr){
 }
 
 UHD_STATIC_BLOCK(register_usrp2_device){
-    device::register_device(&usrp2_find, &usrp2_make);
+    device::register_device(&usrp2_find, &usrp2_make, device::USRP);
 }
 
 /***********************************************************************
@@ -316,9 +316,10 @@ static zero_copy_if::sptr make_xport(
 /***********************************************************************
  * Structors
  **********************************************************************/
-usrp2_impl::usrp2_impl(const device_addr_t &_device_addr){
+usrp2_impl::usrp2_impl(const device_addr_t &_device_addr) :
+    device_addr(_device_addr)
+{
     UHD_MSG(status) << "Opening a USRP2/N-Series device..." << std::endl;
-    device_addr_t device_addr = _device_addr;
 
     //setup the dsp transport hints (default to a large recv buff)
     if (not device_addr.has_key("recv_buff_size")){
@@ -368,6 +369,8 @@ usrp2_impl::usrp2_impl(const device_addr_t &_device_addr){
     // create controller objects and initialize the properties tree
     ////////////////////////////////////////////////////////////////////
     _tree = property_tree::make();
+    _type = device::USRP;
+    _ignore_cal_file = device_addr.has_key("ignore-cal-file");
     _tree->create<std::string>("/name").set("USRP2 / N-Series Device");
 
     for (size_t mbi = 0; mbi < device_args.size(); mbi++){
@@ -392,14 +395,35 @@ usrp2_impl::usrp2_impl(const device_addr_t &_device_addr){
             fpga_major = fpga_minor;
             fpga_minor = 0;
         }
-        if (fpga_major != USRP2_FPGA_COMPAT_NUM){
+        int expected_fpga_compat_num = std::min(USRP2_FPGA_COMPAT_NUM, N200_FPGA_COMPAT_NUM);
+        switch (_mbc[mb].iface->get_rev())
+        {
+        case usrp2_iface::USRP2_REV3:
+        case usrp2_iface::USRP2_REV4:
+            expected_fpga_compat_num = USRP2_FPGA_COMPAT_NUM;
+            break;
+        case usrp2_iface::USRP_N200:
+        case usrp2_iface::USRP_N200_R4:
+        case usrp2_iface::USRP_N210:
+        case usrp2_iface::USRP_N210_R4:
+            expected_fpga_compat_num = N200_FPGA_COMPAT_NUM;
+            break;
+        default:
+            // handle case where the MB EEPROM is not programmed
+            if (fpga_major == USRP2_FPGA_COMPAT_NUM or fpga_major == N200_FPGA_COMPAT_NUM)
+            {
+                UHD_MSG(warning)  << "Unable to identify device - assuming USRP2/N-Series device" << std::endl;
+                expected_fpga_compat_num = fpga_major;
+            }
+        }
+        if (fpga_major != expected_fpga_compat_num){
             throw uhd::runtime_error(str(boost::format(
                 "\nPlease update the firmware and FPGA images for your device.\n"
                 "See the application notes for USRP2/N-Series for instructions.\n"
                 "Expected FPGA compatibility number %d, but got %d:\n"
                 "The FPGA build is not compatible with the host code build.\n"
                 "%s\n"
-            ) % int(USRP2_FPGA_COMPAT_NUM) % fpga_major % _mbc[mb].iface->images_warn_help_message()));
+            ) % expected_fpga_compat_num % fpga_major % _mbc[mb].iface->images_warn_help_message()));
         }
         _tree->create<std::string>(mb_path / "fpga_version").set(str(boost::format("%u.%u") % fpga_major % fpga_minor));
 
@@ -621,9 +645,9 @@ usrp2_impl::usrp2_impl(const device_addr_t &_device_addr){
             .coerce(boost::bind(&tx_dsp_core_200::set_host_rate, _mbc[mb].tx_dsp, _1))
             .subscribe(boost::bind(&usrp2_impl::update_tx_samp_rate, this, mb, 0, _1));
         _tree->create<double>(mb_path / "tx_dsps/0/freq/value")
-            .coerce(boost::bind(&usrp2_impl::set_tx_dsp_freq, this, mb, _1));
+            .coerce(boost::bind(&tx_dsp_core_200::set_freq, _mbc[mb].tx_dsp, _1));
         _tree->create<meta_range_t>(mb_path / "tx_dsps/0/freq/range")
-            .publish(boost::bind(&usrp2_impl::get_tx_dsp_freq_range, this, mb));
+            .publish(boost::bind(&tx_dsp_core_200::get_freq_range, _mbc[mb].tx_dsp));
 
         //setup dsp flow control
         const double ups_per_sec = device_args_i.cast<double>("ups_per_sec", 20);
@@ -758,9 +782,6 @@ usrp2_impl::usrp2_impl(const device_addr_t &_device_addr){
             UHD_MSG(status) << "Setting references to the internal GPSDO" << std::endl;
             _tree->access<std::string>(root / "time_source/value").set("gpsdo");
             _tree->access<std::string>(root / "clock_source/value").set("gpsdo");
-            UHD_MSG(status) << "Initializing time to the internal GPSDO" << std::endl;
-            const time_t tp = time_t(_mbc[mb].gps->get_sensor("gps_time").to_int()+1);
-            _tree->access<time_spec_t>(root / "time" / "pps").set(time_spec_t(tp));
         }
     }
 
@@ -793,39 +814,20 @@ sensor_value_t usrp2_impl::get_ref_locked(const std::string &mb){
 }
 
 void usrp2_impl::set_rx_fe_corrections(const std::string &mb, const double lo_freq){
-    apply_rx_fe_corrections(this->get_tree()->subtree("/mboards/" + mb), "A", lo_freq);
+    if(not _ignore_cal_file){
+        apply_rx_fe_corrections(this->get_tree()->subtree("/mboards/" + mb), "A", lo_freq);
+    }
 }
 
 void usrp2_impl::set_tx_fe_corrections(const std::string &mb, const double lo_freq){
-    apply_tx_fe_corrections(this->get_tree()->subtree("/mboards/" + mb), "A", lo_freq);
+    if(not _ignore_cal_file){
+        apply_tx_fe_corrections(this->get_tree()->subtree("/mboards/" + mb), "A", lo_freq);
+    }
 }
 
 #include <boost/math/special_functions/round.hpp>
 #include <boost/math/special_functions/sign.hpp>
 
-double usrp2_impl::set_tx_dsp_freq(const std::string &mb, const double freq_){
-    double new_freq = freq_;
-    const double tick_rate = _tree->access<double>("/mboards/"+mb+"/tick_rate").get();
-
-    //calculate the DAC shift (multiples of rate)
-    const int sign = boost::math::sign(new_freq);
-    const int zone = std::min(boost::math::iround(new_freq/tick_rate), 2);
-    const double dac_shift = sign*zone*tick_rate;
-    new_freq -= dac_shift; //update FPGA DSP target freq
-
-    //set the DAC shift (modulation mode)
-    if (zone == 0) _mbc[mb].codec->set_tx_mod_mode(0); //no shift
-    else _mbc[mb].codec->set_tx_mod_mode(sign*4/zone); //DAC interp = 4
-
-    return _mbc[mb].tx_dsp->set_freq(new_freq) + dac_shift; //actual freq
-}
-
-meta_range_t usrp2_impl::get_tx_dsp_freq_range(const std::string &mb){
-    const double tick_rate = _tree->access<double>("/mboards/"+mb+"/tick_rate").get();
-    const meta_range_t dsp_range = _mbc[mb].tx_dsp->get_freq_range();
-    return meta_range_t(dsp_range.start() - tick_rate*2, dsp_range.stop() + tick_rate*2, dsp_range.step());
-}
-
 void usrp2_impl::update_clock_source(const std::string &mb, const std::string &source){
     //NOTICE: U2_REG_MISC_CTRL_CLOCK is on the wb clock, and cannot be set from fifo_ctrl
     //clock source ref 10mhz
