@@ -1,5 +1,5 @@
 //
-// Copyright 2013-2014 Ettus Research LLC
+// Copyright 2013-2015 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -45,20 +45,16 @@
 #include <uhd/device.hpp>
 #include <uhd/types/device_addr.hpp>
 #include <uhd/utils/byteswap.hpp>
-#include <uhd/utils/images.hpp>
+#include <uhd/utils/paths.hpp>
 #include <uhd/utils/safe_main.hpp>
 #include <uhd/utils/safe_call.hpp>
 
-#ifdef _MSC_VER
 extern "C" {
-#endif
 #include "cdecode.h"
-#ifdef _MSC_VER
 }
-#endif
 
 #define X300_FPGA_BIN_SIZE_BYTES 15877916
-#define X300_FPGA_BIT_MAX_SIZE_BYTES 15878022
+#define X300_FPGA_BIT_MAX_SIZE_BYTES 15878032
 #define X300_FPGA_PROG_UDP_PORT 49157
 #define X300_FLASH_SECTOR_SIZE 131072
 #define X300_PACKET_SIZE_BYTES 256
@@ -118,7 +114,7 @@ boost::uint8_t bitswap(uint8_t b){
 }
 
 void list_usrps(){
-    device_addrs_t found_devices = device::find(device_addr_t("type=x300"));
+    device_addrs_t found_devices = device::find(device_addr_t("type=x300"), device::USRP);
 
     std::cout << "Available X3x0 devices:" << std::endl;
     BOOST_FOREACH(const device_addr_t &dev, found_devices){
@@ -142,7 +138,7 @@ void list_usrps(){
 device_addr_t find_usrp_with_ethernet(std::string ip_addr, bool output){
     if(output) std::cout << "Attempting to find X3x0 with IP address: " << ip_addr << std::endl;
     const device_addr_t dev = device_addr_t(str(boost::format("addr=%s") % ip_addr));
-    device_addrs_t found_devices = device::find(dev);
+    device_addrs_t found_devices = device::find(dev, device::USRP);
 
     if(found_devices.size() < 1) {
         throw std::runtime_error("Could not find X3x0 with the specified address!");
@@ -161,7 +157,7 @@ device_addr_t find_usrp_with_ethernet(std::string ip_addr, bool output){
 device_addr_t find_usrp_with_pcie(std::string resource, bool output){
     if(output) std::cout << "Attempting to find X3x0 with resource: " << resource << std::endl;
     const device_addr_t dev = device_addr_t(str(boost::format("resource=%s") % resource));
-    device_addrs_t found_devices = device::find(dev);
+    device_addrs_t found_devices = device::find(dev, device::USRP);
 
     if(found_devices.size() < 1) {
         throw std::runtime_error("Could not find X3x0 with the specified resource!");
@@ -199,6 +195,71 @@ void extract_from_lvbitx(std::string lvbitx_path, std::vector<char> &bitstream){
     bitstream.swap(decoded_bitstream);
 }
 
+void print_image_loader_warning(const std::string &fpga_path, const po::variables_map &vm){
+
+    // Newline + indent
+    #ifdef UHD_PLATFORM_WIN32
+    const std::string nl = " ^\n    ";
+    #else
+    const std::string nl = " \\\n    ";
+    #endif
+
+    // Generate equivalent uhd_image_loader command
+    std::string uhd_image_loader = "uhd_image_loader --args=\"type=x300";
+
+    if(vm.count("addr") > 0){
+        uhd_image_loader += str(boost::format(",addr=%s")
+                                % vm["addr"].as<std::string>());
+
+        if(vm.count("configure") > 0){
+            uhd_image_loader += ",configure";
+        }
+
+        if(vm.count("verify") > 0){
+            uhd_image_loader += ",verify";
+        }
+    }
+    else{
+        uhd_image_loader += str(boost::format(",resource=%s")
+                                % vm["resource"].as<std::string>());
+
+        /*
+         * Since we have a default value, vm.count("rpc-port") will
+         * always be > 0, so only add the option if a different port
+         * is given.
+         */
+        if(vm["rpc-port"].as<std::string>() != "5444"){
+            uhd_image_loader += str(boost::format(",rpc-port=%s")
+                                    % vm["rpc-port"].as<std::string>());
+        }
+    }
+
+    if(vm.count("type") > 0){
+        uhd_image_loader += str(boost::format(",fpga=%s")
+                                % vm["type"].as<std::string>());
+    }
+
+    uhd_image_loader += "\"";
+
+    /*
+     * The --type option overrides any given path, so only add an FPGA path
+     * if there was no --type argument.
+     */
+    if(vm.count("type") == 0){
+        uhd_image_loader += str(boost::format("%s--fpga-path=\"%s\"")
+                                % nl % fpga_path);
+    }
+
+    std::cout << "************************************************************************************************" << std::endl
+              << "WARNING: This utility will be removed in an upcoming version of UHD. In the future, use" << std::endl
+              << "         this command:" << std::endl
+              << std::endl
+              << uhd_image_loader << std::endl
+              << std::endl
+              << "************************************************************************************************" << std::endl
+              << std::endl;
+}
+
 void ethernet_burn(udp_simple::sptr udp_transport, std::string fpga_path, bool verify){
     boost::uint32_t max_size;
     std::vector<char> bitstream;
@@ -483,11 +544,13 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
         throw std::runtime_error("The image filename must end in .bin, .bit, or .lvbitx.");
     }
 
+    print_image_loader_warning(fpga_path, vm);
+
     std::signal(SIGINT, &sig_int_handler);
     if(vm.count("addr")){
         udp_simple::sptr udp_transport = udp_simple::make_connected(ip_addr, BOOST_STRINGIZE(X300_FPGA_PROG_UDP_PORT));
 
-        ethernet_burn(udp_transport, fpga_path, vm.count("verify"));
+        ethernet_burn(udp_transport, fpga_path, (vm.count("verify") > 0));
 
         if(vm.count("configure")){
             if(configure_fpga(udp_transport, ip_addr)) std::cout << "Successfully configured FPGA!" << std::endl;
